/**
 * Sort Service
 */
import {Injectable} from '@angular/core';

// Services
import {AppConfigService} from 'wcax-core-angular/app.config.service';
import * as _ from 'lodash';

// Data model
import {ProductModel, SortModel, CouponModel} from '../data.model';

@Injectable()
export class SortService {
    private sorts: SortModel[] = new Array<SortModel>();
    private available: string[];
    private default: SortModel;
    public rules;

    private initRules() {
        let results: { [type: string]: SortModel };
        results = {
            'a-z': {
                field: 'name',
                rule: [{field: 'name', direction: -1, type: 'string'}],
                label: 'Alphabetically'
            },
            'a-z-description': {
                field: 'description',
                rule: [{field: 'description', direction: -1, type: 'string'}],
                label: 'Alphabetically'
            },
            'search-rank': {
                field: 'searchRank',
                rule: [
                    {field: 'searchRank', direction: -1, type: 'number'},
                    {field: 'name', direction: -1, type: 'string'}
                ],
                label: 'Search Ranking'
            },
            'most-popular': {
                field: 'salesRank',
                rule: [
                    {field: 'salesRank', direction: 1, type: 'number'},
                    {field: 'name', direction: -1, type: 'string'}
                ],
                label: 'Most Popular'
            },
            'price-low-high': {
                field: 'price',
                rule: [
                    {field: 'price', direction: -1, type: 'number'},
                    {field: 'name', direction: -1, type: 'string'}
                ],
                label: 'Price - Low to High'
            },
            'purchase-history': {
                field: 'purchaseRank',
                rule: [
                    {field: 'purchaseRank', direction: -1, type: 'string'}
                ],
                label: 'Purchase History'
            },
            'new-offers': {
                field: 'arrivalRank',
                rule: [
                    {field: 'arrivalRank', direction: 1, type: 'string'}
                ],
                label: 'New Offers'
            },
            'about-to-expire': {
                field: 'expiryRank',
                rule: [
                    {field: 'expiryRank', direction: 1, type: 'string'}
                ],
                label: 'About to Expire'
            },
            'category': {
                field: 'categoryId',
                rule: [
                    {field: 'categoryId', direction: 1, type: 'string'}
                ],
                label: 'Category'
            },
            'expiry-date': {
                field: 'endDate',
                rule: [
                    {field: 'endDate', direction: 1, type: 'string'}
                ],
                label: 'Expiry Date'
            },
            'added-date': {
                field: 'addedDate',
                rule: [
                    {field: 'addedDate', direction: 1, type: 'string'}
                ],
                label: 'Added Date'
            },
            'reward-card-specials': {
                field: 'promoDescription',
                rule: [
                    {
                        field: 'promoDescription',
                        direction: -1,
                        type: 'custom',
                        callback: function (tempA, sort) {
                            if (this.isPromoMatch(tempA[sort.field])) {
                                return 'AAA';
                            } else {
                                return 'ZZZ';
                            }
                        }
                    },
                    {field: 'name', direction: -1, type: 'string'}
                ],
                label: this.appConfigService.getPromoDescription()
            },
            aisles: {
                // dummy, needs to be here for now for backwards compatibility
                field: 'groupByTemplate',
                rule: [],
                label: 'Aisles'
            },
            aisles_departmentName: {
                field: 'departmentName',
                rule: [
                    {field: 'departmentName', direction: -1, type: 'string'},
                    {field: 'name', direction: -1, type: 'string'}
                ],
                label: 'Aisles'
            },
            aisles_aisleName: {
                field: 'aisleName',
                rule: [
                    {field: 'aisleName', direction: -1, type: 'string'},
                    {field: 'name', direction: -1, type: 'string'}
                ],
                label: 'Aisles'
            },
            aisles_shelfName: {
                field: 'shelfName',
                rule: [
                    {field: 'shelfName', direction: -1, type: 'string'},
                    {field: 'name', direction: -1, type: 'string'}
                ],
                label: 'Aisles'
            },
            'bloomreach-top-sellers': {
                field: 'salesRank+asc',
                rule: null,
                label: 'Top sellers'
            },
            'bloomreach-price-low-high': {
                field: 'price+asc',
                rule: null,
                label: 'Price low to high'
            },
            'bloomreach-best-match': {
                field: '',
                rule: null,
                label: 'Best Match'
            },
            'bloomreach-club-card': {
                field: 'promoType+asc',
                rule: null,
                label: this.appConfigService.getPromoDescription()
            },
            aisles_coupons: {
                // dummy, needs to be here for now for backwards compatibility
                field: 'purchaseInd',
                rule: [{field: 'purchaseInd', direction: -1, type: 'string'}],
                label: 'Type of Coupons'
            }
        };
        return results;
    }

    private isPromoMatch(label: string): boolean {
        const promoRules = this.appConfigService.getPromorulesArray();


        if (!promoRules || promoRules.length === 0) {
            return false;
        }

        let found = false;
        promoRules.forEach(function (promoRule) {
            if (label && label.length > 0) {
                if (label.match(promoRule['rule'])) {
                    found = true;
                }
            }
        });

        return found;
    }

    public constructor(public appConfigService: AppConfigService) {
        _.padStart('Hello TypeScript!', 20, ' ');
        this.rules = this.initRules();
        this.available = window['AB']['availableSorts'] || [];

        if (this.hasSort()) {
            this.generate();
        }
    }

    public setAvailableSort(sort) {
        this.available = sort;
    }

    public getDefaultSort() {
        if (this.default != undefined)
            return this.default;

        if (this.available.filter(rule => rule === 'a-z').length > 0)
            this.default = this.getSortByName('a-z');
        else
            this.default = (this.available[0] != undefined) ? this.getSortByName(this.available[0]) : this.getSortByName('a-z');

        return this.default;
    }

    public getDefaultSortForConfigurableSort(defaultRule) {
        if (this.available.filter(rule => rule === defaultRule).length > 0)
            this.default = this.getSortByName(defaultRule);
        else
            this.default = (this.available[0] != undefined) ? this.getSortByName(this.available[0]) : this.getSortByName(defaultRule);

        return this.default;
    }

    public hasSort() {
        return !!this.available.length;
    }

    public getSortByName(name) {
        return this.rules[name];
    }

    public getSorts() {
        return this.sorts;
    }

    public sort(sort: SortModel, data: ProductModel[]): ProductModel[] {
        const sortByCriteria = sort.rule.map(rule => {
            return (a: ProductModel) => {
                const tempA: ProductModel = Object.assign({}, a);

                if (rule.type === 'string') {
                    if (tempA && tempA[rule.field]) {
                        tempA[rule.field] = tempA[rule.field].toString().toLowerCase();
                    }
                }

                if (rule.type === 'number') {
                    tempA[rule.field] = parseFloat(tempA[rule.field]);
                }

                if (rule.type === 'custom') {
                    return rule.callback.call(this, tempA, sort);
                }

                return tempA[rule.field];
            };
        });

        const sortByDirection = sort.rule.map(rule => {
            if (rule.direction === 1) {
                return 'asc';
            } else {
                return 'desc';
            }
        });
        return <any>_.sortBy(data, sortByCriteria, sortByDirection);
    }

    public couponsSort(sort: SortModel, data: CouponModel[]): CouponModel[] {
        const sortByCriteria = sort.rule.map(rule => {
            return (a: CouponModel) => {
                const tempA: CouponModel = Object.assign({}, a);

                if (rule.type === 'string') {
                    if (tempA && tempA[rule.field]) {
                        tempA[rule.field] = tempA[rule.field].toString().toLowerCase();
                    }
                }

                if (rule.type === 'number') {
                    tempA[rule.field] = parseFloat(tempA[rule.field]);
                }

                if (rule.type === 'custom') {
                    return rule.callback.call(this, tempA, sort);
                }

                return tempA[rule.field];
            };
        });

        const sortByDirection = sort.rule.map(rule => {
            if (rule.direction === 1) {
                return 'asc';
            } else {
                return 'desc';
            }
        });

        return <any>_.sortBy(data, sortByCriteria, sortByDirection);
    }

    public generate() {
        if (this.available.length === 0) {
            return;
        }

        this.available.forEach((type: string, index: number) => {
            let rule = this.rules[type];

            if (!rule) {
                return;
            }

            /* overwrite with custom group by rule based on headerType setting */
            if (rule.field === 'groupByTemplate') {
                const groupByRuleKey = 'aisles' + '_' + window['AB']['headerType'];
                rule.rule = this.rules[groupByRuleKey].rule;
            }
            this.sorts.push(new SortModel(rule));

            if (this.default === undefined && type === window['AB']['defaultSort']) {
                this.default = rule;
            }

        });
    }
}
